@@ -34,6 +34,8 @@
 #include "base/version.h"
 #include "client/client.h"
 #include "ipc/ipc_mock.h"
+#include "languages/global_language_spec.h"
+#include "languages/japanese/lang_dep_spec.h"
 #include "session/commands.pb.h"
 #include "testing/base/public/gunit.h"
 
@@ -166,6 +168,9 @@ class ClientTest : public testing::Test {
   ClientTest() : version_diff_(0) {}
 
   virtual void SetUp() {
+    mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(
+        &language_dependent_spec_japanese_);
+
     client_factory_.reset(new IPCClientFactoryMock);
     client_.reset(new Client);
     client_->SetIPCClientFactory(client_factory_.get());
@@ -177,6 +182,7 @@ class ClientTest : public testing::Test {
   virtual void TearDown() {
     client_.reset();
     client_factory_.reset();
+    mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(NULL);
   }
 
   void SetMockOutput(const commands::Output &mock_output) {
@@ -218,6 +224,9 @@ class ClientTest : public testing::Test {
   int version_diff_;
 
  private:
+  // We need to set a LangDepSpecJapanese to GlobalLanguageSpec on start up for
+  // testing, and the actual instance does not have to be LangDepSpecJapanese.
+  mozc::japanese::LangDepSpecJapanese language_dependent_spec_japanese_;
   DISALLOW_COPY_AND_ASSIGN(ClientTest);
 };
 
@@ -632,6 +641,9 @@ class SessionPlaybackTest : public testing::Test {
   virtual ~SessionPlaybackTest() {}
 
   virtual void SetUp() {
+    mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(
+        &language_dependent_spec_japanese_);
+
     ipc_client_factory_.reset(new IPCClientFactoryMock);
     ipc_client_.reset(reinterpret_cast<IPCClientMock *>(
         ipc_client_factory_->NewClient("")));
@@ -645,6 +657,7 @@ class SessionPlaybackTest : public testing::Test {
   virtual void TearDown() {
     client_.reset();
     ipc_client_factory_.reset();
+    mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(NULL);
   }
 
   bool SetupConnection(const int id) {
@@ -678,6 +691,9 @@ class SessionPlaybackTest : public testing::Test {
   SessionPlaybackTestServerLauncher *server_launcher_;
 
  private:
+  // We need to set a LangDepSpecJapanese to GlobalLanguageSpec on start up for
+  // testing, and the actual instance does not have to be LangDepSpecJapanese.
+  mozc::japanese::LangDepSpecJapanese language_dependent_spec_japanese_;
   DISALLOW_COPY_AND_ASSIGN(SessionPlaybackTest);
 };
 
