@@ -27,12 +27,16 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-#include "converter/character_form_manager.h"
 #include "composer/composer.h"
+
+#include <string>
+#include <utility>
+
+#include "converter/character_form_manager.h"
 #include "composer/table.h"
+#include "config/config_handler.h"
+#include "config/config.pb.h"
 #include "session/commands.pb.h"
-#include "session/config_handler.h"
-#include "session/config.pb.h"
 #include "session/key_parser.h"
 #include "testing/base/public/gunit.h"
 
@@ -67,6 +71,19 @@ string GetPreedit(const Composer *composer) {
   return preedit;
 }
 
+void ExpectSameComposer(const Composer &lhs, const Composer &rhs) {
+  // TODO(hsumita): Check composition_
+  EXPECT_EQ(lhs.GetCursor(), rhs.GetCursor());
+  EXPECT_EQ(lhs.is_new_input(), rhs.is_new_input());
+  EXPECT_EQ(lhs.GetInputMode(), rhs.GetInputMode());
+  EXPECT_EQ(lhs.GetOutputMode(), rhs.GetOutputMode());
+  EXPECT_EQ(lhs.GetComebackInputMode(), rhs.GetComebackInputMode());
+  EXPECT_EQ(lhs.shifted_sequence_count(), rhs.shifted_sequence_count());
+  EXPECT_EQ(lhs.source_text(), rhs.source_text());
+  EXPECT_EQ(lhs.max_length(), rhs.max_length());
+  EXPECT_EQ(lhs.GetInputFieldType(), rhs.GetInputFieldType());
+}
+
 }  // namespace
 
 class ComposerTest : public testing::Test {
@@ -105,11 +122,14 @@ TEST_F(ComposerTest, Reset) {
   composer_->InsertCharacter("mozuku");
 
   composer_->SetInputMode(transliteration::HALF_ASCII);
+  composer_->SetInputFieldType(commands::SessionCommand::PASSWORD);
   composer_->Reset();
 
   EXPECT_TRUE(composer_->Empty());
   // The input mode ramains the previous mode.
   EXPECT_EQ(transliteration::HALF_ASCII, composer_->GetInputMode());
+  EXPECT_EQ(commands::SessionCommand::PASSWORD,
+            composer_->GetInputFieldType());
 }
 
 TEST_F(ComposerTest, ResetInputMode) {
@@ -186,7 +206,6 @@ TEST_F(ComposerTest, BackSpace) {
   EXPECT_EQ("ab", result);
 }
 
-
 TEST_F(ComposerTest, InsertCharacterPreeditAt) {
   // "あ"
   table_->AddRule("a", "\xe3\x81\x82", "");
@@ -509,7 +528,6 @@ TEST_F(ComposerTest, GetSubTransliterations) {
             transliterations[transliteration::HALF_KATAKANA]);
 }
 
-
 TEST_F(ComposerTest, GetStringFunctions) {
   // "か"
   table_->AddRule("ka", "\xe3\x81\x8b", "");
@@ -589,7 +607,7 @@ TEST_F(ComposerTest, GetStringFunctions) {
 
   prediction.clear();
   composer_->GetQueryForPrediction(&prediction);
-  EXPECT_TRUE(prediction.empty());
+  EXPECT_EQ("s", prediction);
 
   // Query: "sk"
   composer_->EditErase();
@@ -656,6 +674,11 @@ TEST_F(ComposerTest, GetStringFunctions_ForN) {
   EXPECT_EQ("ny[NYA][N][KA]", prediction2);
 }
 
+TEST_F(ComposerTest, InsertCommandCharacter) {
+  composer_->SetInputMode(transliteration::HALF_ASCII);
+  composer_->InsertCommandCharacter(Composer::REWIND);
+  EXPECT_EQ("\x0F<\x0E", GetPreedit(composer_.get()));
+}
 
 TEST_F(ComposerTest, InsertCharacterKeyEvent) {
   commands::KeyEvent key;
@@ -695,7 +718,6 @@ TEST_F(ComposerTest, InsertCharacterKeyEvent) {
   // "あ"
   EXPECT_EQ("\xE3\x81\x82", preedit);
 
-
   // Typing "A" temporarily switch the input mode.  The input mode
   // should be reverted back after reset.
   composer_->SetInputMode(transliteration::FULL_KATAKANA);
@@ -721,7 +743,6 @@ TEST_F(ComposerTest, InsertCharacterKeyEvent) {
   EXPECT_EQ("\xE3\x82\xA2", preedit);
 }
 
-
 TEST_F(ComposerTest, InsertCharacterKeyEventWithAsIs) {
   commands::KeyEvent key;
   // "あ"
@@ -874,6 +895,297 @@ TEST_F(ComposerTest, InsertCharacterKeyEventWithInputMode) {
   }
 }
 
+TEST_F(ComposerTest, ApplyTemporaryInputMode) {
+  const bool kCapsLocked = true;
+  const bool kCapsUnlocked = false;
+
+  // "あ"
+  table_->AddRule("a", "\xE3\x81\x82", "");
+  composer_->SetInputMode(transliteration::HIRAGANA);
+
+  // Since handlings of continuous shifted input differ,
+  // test cases differ between ASCII_INPUT_MODE and KATAKANA_INPUT_MODE
+
+  {  // ASCII_INPUT_MODE (w/o CapsLock)
+    config::Config config;
+    config::ConfigHandler::GetConfig(&config);
+    config.set_shift_key_mode_switch(config::Config::ASCII_INPUT_MODE);
+    config::ConfigHandler::SetConfig(config);
+
+    // pair<input, use_temporary_input_mode>
+    pair<string, bool> kTestDataAscii[] = {
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair(".", true),
+      make_pair("a", true),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair(".", true),
+      make_pair("a", true),
+      // "あ"
+      make_pair("\xE3\x81\x82", false),
+      make_pair("a", false),
+    };
+
+    for (int i = 0; i < arraysize(kTestDataAscii); ++i) {
+      composer_->ApplyTemporaryInputMode(kTestDataAscii[i].first,
+                                         kCapsUnlocked);
+
+      const transliteration::TransliterationType expected =
+          kTestDataAscii[i].second
+          ? transliteration::HALF_ASCII
+          : transliteration::HIRAGANA;
+
+      EXPECT_EQ(expected, composer_->GetInputMode()) << "index=" << i;
+      EXPECT_EQ(transliteration::HIRAGANA, composer_->GetComebackInputMode())
+          << "index=" << i;
+    }
+  }
+
+  {  // ASCII_INPUT_MODE (w/ CapsLock)
+    config::Config config;
+    config::ConfigHandler::GetConfig(&config);
+    config.set_shift_key_mode_switch(config::Config::ASCII_INPUT_MODE);
+    config::ConfigHandler::SetConfig(config);
+
+    // pair<input, use_temporary_input_mode>
+    pair<string, bool> kTestDataAscii[] = {
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair(".", true),
+      make_pair("A", true),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair(".", true),
+      make_pair("A", true),
+      // "あ"
+      make_pair("\xE3\x81\x82", false),
+      make_pair("A", false),
+    };
+
+    for (int i = 0; i < arraysize(kTestDataAscii); ++i) {
+      composer_->ApplyTemporaryInputMode(kTestDataAscii[i].first,
+                                         kCapsLocked);
+
+      const transliteration::TransliterationType expected =
+          kTestDataAscii[i].second
+          ? transliteration::HALF_ASCII
+          : transliteration::HIRAGANA;
+
+      EXPECT_EQ(expected, composer_->GetInputMode()) << "index=" << i;
+      EXPECT_EQ(transliteration::HIRAGANA, composer_->GetComebackInputMode())
+          << "index=" << i;
+    }
+  }
+
+  {  // KATAKANA_INPUT_MODE (w/o CapsLock)
+    config::Config config;
+    config::ConfigHandler::GetConfig(&config);
+    config.set_shift_key_mode_switch(config::Config::KATAKANA_INPUT_MODE);
+    config::ConfigHandler::SetConfig(config);
+
+    // pair<input, use_temporary_input_mode>
+    pair<string, bool> kTestDataKatakana[] = {
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair("a", false),
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair(".", true),
+      make_pair("a", false),
+      make_pair("A", true),
+      make_pair("A", true),
+      make_pair(".", true),
+      make_pair("a", false),
+      // "あ"
+      make_pair("\xE3\x81\x82", false),
+      make_pair("a", false),
+    };
+
+    for (int i = 0; i < arraysize(kTestDataKatakana); ++i) {
+      composer_->ApplyTemporaryInputMode(kTestDataKatakana[i].first,
+                                         kCapsUnlocked);
+
+      const transliteration::TransliterationType expected =
+          kTestDataKatakana[i].second
+          ? transliteration::FULL_KATAKANA
+          : transliteration::HIRAGANA;
+
+      EXPECT_EQ(expected, composer_->GetInputMode()) << "index=" << i;
+      EXPECT_EQ(transliteration::HIRAGANA, composer_->GetComebackInputMode())
+          << "index=" << i;
+    }
+  }
+
+  {  // KATAKANA_INPUT_MODE (w/ CapsLock)
+    config::Config config;
+    config::ConfigHandler::GetConfig(&config);
+    config.set_shift_key_mode_switch(config::Config::KATAKANA_INPUT_MODE);
+    config::ConfigHandler::SetConfig(config);
+
+    // pair<input, use_temporary_input_mode>
+    pair<string, bool> kTestDataKatakana[] = {
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair("A", false),
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair(".", true),
+      make_pair("A", false),
+      make_pair("a", true),
+      make_pair("a", true),
+      make_pair(".", true),
+      make_pair("A", false),
+      // "あ"
+      make_pair("\xE3\x81\x82", false),
+      make_pair("A", false),
+    };
+
+    for (int i = 0; i < arraysize(kTestDataKatakana); ++i) {
+      composer_->ApplyTemporaryInputMode(kTestDataKatakana[i].first,
+                                         kCapsLocked);
+
+      const transliteration::TransliterationType expected =
+          kTestDataKatakana[i].second
+          ? transliteration::FULL_KATAKANA
+          : transliteration::HIRAGANA;
+
+      EXPECT_EQ(expected, composer_->GetInputMode()) << "index=" << i;
+      EXPECT_EQ(transliteration::HIRAGANA, composer_->GetComebackInputMode())
+          << "index=" << i;
+    }
+  }
+}
+
+TEST_F(ComposerTest, CopyFrom) {
+  // "あ"
+  table_->AddRule("a", "\xE3\x81\x82", "");
+  // "ん"
+  table_->AddRule("n", "\xE3\x82\x93", "");
+  // "な"
+  table_->AddRule("na", "\xE3\x81\xAA", "");
+
+  {  // for Precomposition
+    string src_composition;
+    composer_->GetStringForSubmission(&src_composition);
+    EXPECT_EQ("", src_composition);
+
+    Composer dest;
+    string dest_composition;
+    dest.CopyFromForSubmission(*composer_);
+    dest.GetStringForSubmission(&dest_composition);
+    EXPECT_EQ(dest_composition, src_composition);
+
+    ExpectSameComposer(*composer_, dest);
+  }
+
+  {  // for Composition
+    composer_->InsertCharacter("a");
+    composer_->InsertCharacter("n");
+    string src_composition;
+    composer_->GetStringForSubmission(&src_composition);
+    // "あｎ"
+    EXPECT_EQ("\xE3\x81\x82\xEF\xBD\x8E", src_composition);
+
+    Composer dest;
+    string dest_composition;
+    dest.CopyFromForSubmission(*composer_);
+    dest.GetStringForSubmission(&dest_composition);
+    EXPECT_EQ(dest_composition, src_composition);
+
+    ExpectSameComposer(*composer_, dest);
+  }
+
+  {  // for Conversion
+    string src_composition;
+    composer_->GetQueryForConversion(&src_composition);
+    // "あん"
+    EXPECT_EQ("\xE3\x81\x82\xE3\x82\x93", src_composition);
+
+    Composer dest;
+    string dest_composition;
+    dest.CopyFromForConversion(*composer_);
+    dest.GetStringForSubmission(&dest_composition);
+    EXPECT_EQ(dest_composition, src_composition);
+
+    ExpectSameComposer(*composer_, dest);
+  }
+
+  {  // for Composition, temporary input mode
+    composer_->Reset();
+    InsertKey("A", composer_.get());
+    InsertKey("a", composer_.get());
+    InsertKey("A", composer_.get());
+    InsertKey("A", composer_.get());
+    InsertKey("a", composer_.get());
+    string src_composition;
+    composer_->GetStringForSubmission(&src_composition);
+    // "AaAAあ"
+    EXPECT_EQ("AaAA\xE3\x81\x82", src_composition);
+
+    Composer dest;
+    string dest_composition;
+    dest.CopyFromForSubmission(*composer_);
+    dest.GetStringForSubmission(&dest_composition);
+    EXPECT_EQ(dest_composition, src_composition);
+
+    ExpectSameComposer(*composer_, dest);
+  }
+
+  {  // for Composition, password mode
+    composer_->Reset();
+    composer_->SetInputFieldType(commands::SessionCommand::PASSWORD);
+    composer_->SetInputMode(transliteration::HALF_ASCII);
+    composer_->SetOutputMode(transliteration::HALF_ASCII);
+    composer_->InsertCharacter("M");
+    string src_composition;
+    composer_->GetStringForSubmission(&src_composition);
+    EXPECT_EQ("M", src_composition);
+
+    Composer dest;
+    string dest_composition;
+    dest.CopyFromForSubmission(*composer_);
+    dest.GetStringForSubmission(&dest_composition);
+    EXPECT_EQ(dest_composition, src_composition);
+
+    ExpectSameComposer(*composer_, dest);
+  }
+}
+
 TEST_F(ComposerTest, ShiftKeyOperation) {
   commands::KeyEvent key;
   // "あ"
@@ -2225,7 +2537,7 @@ TEST_F(ComposerTest,
   }
 }
 
-TEST_F(ComposerTest, CursorMoving) {
+TEST_F(ComposerTest, InputModesChangeWhenCursorMoves) {
   // The expectation of this test is the same as MS-IME's
 
   table_->Initialize();
@@ -2386,5 +2698,206 @@ TEST_F(ComposerTest, ShuoldCommit) {
   EXPECT_FALSE(composer_->ShouldCommit());
 }
 
+TEST_F(ComposerTest, ShouldCommitHead) {
+  struct TestData {
+    const string input_text;
+    const commands::SessionCommand::InputFieldType field_type;
+    const bool expected_return;
+    const size_t expected_commit_length;
+    TestData(const string &input_text,
+             commands::SessionCommand::InputFieldType field_type,
+             bool expected_return,
+             size_t expected_commit_length)
+        : input_text(input_text),
+          field_type(field_type),
+          expected_return(expected_return),
+          expected_commit_length(expected_commit_length) {
+    }
+  };
+  const TestData test_data_list[] = {
+      // On NORMAL, never commit the head.
+      TestData("", commands::SessionCommand::NORMAL, false, 0),
+      TestData("A", commands::SessionCommand::NORMAL, false, 0),
+      TestData("AB", commands::SessionCommand::NORMAL, false, 0),
+      TestData("", commands::SessionCommand::PASSWORD, false, 0),
+      // On PASSWORD, commit (length - 1) characters.
+      TestData("A", commands::SessionCommand::PASSWORD, false, 0),
+      TestData("AB", commands::SessionCommand::PASSWORD, true, 1),
+      TestData("ABCDEFGHI", commands::SessionCommand::PASSWORD, true, 8),
+      // On NUMBER and TEL, commit (length) characters.
+      TestData("", commands::SessionCommand::NUMBER, false, 0),
+      TestData("A", commands::SessionCommand::NUMBER, true, 1),
+      TestData("AB", commands::SessionCommand::NUMBER, true, 2),
+      TestData("ABCDEFGHI", commands::SessionCommand::NUMBER, true, 9),
+      TestData("", commands::SessionCommand::TEL, false, 0),
+      TestData("A", commands::SessionCommand::TEL, true, 1),
+      TestData("AB", commands::SessionCommand::TEL, true, 2),
+      TestData("ABCDEFGHI", commands::SessionCommand::TEL, true, 9),
+  };
+
+  for (size_t i = 0; i < ARRAYSIZE(test_data_list); ++i) {
+    const TestData &test_data = test_data_list[i];
+    SCOPED_TRACE(test_data.input_text);
+    SCOPED_TRACE(test_data.field_type);
+    SCOPED_TRACE(test_data.expected_return);
+    SCOPED_TRACE(test_data.expected_commit_length);
+    composer_->Reset();
+    composer_->SetInputFieldType(test_data.field_type);
+    composer_->InsertCharacter(test_data.input_text);
+    size_t length_to_commit;
+    const bool result = composer_->ShouldCommitHead(&length_to_commit);
+    if (test_data.expected_return) {
+      EXPECT_TRUE(result);
+      EXPECT_EQ(test_data.expected_commit_length, length_to_commit);
+    } else {
+      EXPECT_FALSE(result);
+    }
+  }
+}
+
+TEST_F(ComposerTest, CursorMovements) {
+  composer_->InsertCharacter("mozuku");
+  EXPECT_EQ(6, composer_->GetLength());
+  EXPECT_EQ(6, composer_->GetCursor());
+
+  composer_->MoveCursorRight();
+  EXPECT_EQ(6, composer_->GetCursor());
+  composer_->MoveCursorLeft();
+  EXPECT_EQ(5, composer_->GetCursor());
+
+  composer_->MoveCursorToBeginning();
+  EXPECT_EQ(0, composer_->GetCursor());
+  composer_->MoveCursorLeft();
+  EXPECT_EQ(0, composer_->GetCursor());
+  composer_->MoveCursorRight();
+  EXPECT_EQ(1, composer_->GetCursor());
+
+  composer_->MoveCursorTo(0);
+  EXPECT_EQ(0, composer_->GetCursor());
+  composer_->MoveCursorTo(6);
+  EXPECT_EQ(6, composer_->GetCursor());
+  composer_->MoveCursorTo(3);
+  EXPECT_EQ(3, composer_->GetCursor());
+  composer_->MoveCursorTo(10);
+  EXPECT_EQ(3, composer_->GetCursor());
+  composer_->MoveCursorTo(-1);
+  EXPECT_EQ(3, composer_->GetCursor());
+}
+
+TEST_F(ComposerTest, SourceText) {
+  composer_->SetInputMode(transliteration::HALF_ASCII);
+  composer_->InsertCharacterPreedit("mozc");
+  composer_->mutable_source_text()->assign("MOZC");
+  EXPECT_FALSE(composer_->Empty());
+  EXPECT_EQ("mozc", GetPreedit(composer_.get()));
+  EXPECT_EQ("MOZC", composer_->source_text());
+
+  composer_->Backspace();
+  composer_->Backspace();
+  EXPECT_FALSE(composer_->Empty());
+  EXPECT_EQ("mo", GetPreedit(composer_.get()));
+  EXPECT_EQ("MOZC", composer_->source_text());
+
+  composer_->Reset();
+  EXPECT_TRUE(composer_->Empty());
+  EXPECT_TRUE(composer_->source_text().empty());
+}
+
+TEST_F(ComposerTest, DeleteAt) {
+  // "も"
+  table_->AddRule("mo", "\xE3\x82\x82", "");
+  // "ず"
+  table_->AddRule("zu", "\xE3\x81\x9a", "");
+
+  composer_->InsertCharacter("z");
+  // "ｚ"
+  EXPECT_EQ("\xef\xbd\x9a", GetPreedit(composer_.get()));
+  EXPECT_EQ(1, composer_->GetCursor());
+  composer_->DeleteAt(0);
+  EXPECT_EQ("", GetPreedit(composer_.get()));
+  EXPECT_EQ(0, composer_->GetCursor());
+
+  composer_->InsertCharacter("mmoz");
+  // "ｍもｚ"
+  EXPECT_EQ("\xef\xbd\x8d\xe3\x82\x82\xef\xbd\x9a",
+            GetPreedit(composer_.get()));
+  EXPECT_EQ(3, composer_->GetCursor());
+  composer_->DeleteAt(0);
+  // "もｚ"
+  EXPECT_EQ("\xe3\x82\x82\xef\xbd\x9a", GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+  composer_->InsertCharacter("u");
+  // "もず"
+  EXPECT_EQ("\xe3\x82\x82\xe3\x81\x9a", GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+
+  composer_->InsertCharacter("m");
+  // "もずｍ"
+  EXPECT_EQ("\xe3\x82\x82\xe3\x81\x9a\xef\xbd\x8d",
+            GetPreedit(composer_.get()));
+  EXPECT_EQ(3, composer_->GetCursor());
+  composer_->DeleteAt(1);
+  // "もｍ"
+  EXPECT_EQ("\xe3\x82\x82\xef\xbd\x8d", GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+  composer_->InsertCharacter("o");
+  // "もも"
+  EXPECT_EQ("\xe3\x82\x82\xe3\x82\x82", GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+}
+
+TEST_F(ComposerTest, DeleteRange) {
+  // "も"
+  table_->AddRule("mo", "\xE3\x82\x82", "");
+  // "ず"
+  table_->AddRule("zu", "\xE3\x81\x9a", "");
+
+  composer_->InsertCharacter("z");
+  // "ｚ"
+  EXPECT_EQ("\xef\xbd\x9a", GetPreedit(composer_.get()));
+  EXPECT_EQ(1, composer_->GetCursor());
+
+  composer_->DeleteRange(0, 1);
+  EXPECT_EQ("", GetPreedit(composer_.get()));
+  EXPECT_EQ(0, composer_->GetCursor());
+
+  composer_->InsertCharacter("mmozmoz");
+  // "ｍもｚもｚ"
+  EXPECT_EQ("\xef\xbd\x8d\xe3\x82\x82\xef\xbd\x9a\xe3\x82\x82\xef\xbd\x9a",
+            GetPreedit(composer_.get()));
+  EXPECT_EQ(5, composer_->GetCursor());
+
+  composer_->DeleteRange(0, 3);
+  // "もｚ"
+  EXPECT_EQ("\xe3\x82\x82\xef\xbd\x9a", GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+
+  composer_->InsertCharacter("u");
+  // "もず"
+  EXPECT_EQ("\xe3\x82\x82\xe3\x81\x9a", GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+
+  composer_->InsertCharacter("xyz");
+  composer_->MoveCursorToBeginning();
+  composer_->InsertCharacter("mom");
+  // "もｍ|もずｘｙｚ"
+  EXPECT_EQ("\xe3\x82\x82\xef\xbd\x8d\xe3\x82\x82\xe3\x81\x9a"
+            "\xef\xbd\x98\xef\xbd\x99\xef\xbd\x9a",
+            GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+
+  composer_->DeleteRange(2, 3);
+  // "もｍ|ｙｚ"
+  EXPECT_EQ("\xe3\x82\x82\xef\xbd\x8d\xef\xbd\x99\xef\xbd\x9a",
+            GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+
+  composer_->InsertCharacter("o");
+  // "もも|ｙｚ"
+  EXPECT_EQ("\xe3\x82\x82\xe3\x82\x82\xef\xbd\x99\xef\xbd\x9a",
+            GetPreedit(composer_.get()));
+  EXPECT_EQ(2, composer_->GetCursor());
+}
+
 }  // namespace composer
 }  // namespace mozc
