@@ -43,12 +43,13 @@
 #include "base/run_level.h"
 #include "base/util.h"
 #include "gui/base/win_util.h"
-#include "client/session.h"
+#include "client/client.h"
 #include "dictionary/user_dictionary_importer.h"
 #include "dictionary/user_dictionary_storage.h"
 #include "dictionary/user_dictionary_util.h"
 #include "dictionary/user_pos.h"
 #include "gui/config_dialog/combobox_delegate.h"
+#include "gui/dictionary_tool/find_dialog.h"
 #include "gui/dictionary_tool/import_dialog.h"
 
 namespace mozc {
@@ -89,11 +90,13 @@ QProgressDialog *CreateProgressDialog(
   return progress;
 }
 
+#if defined(OS_WINDOWS) || defined(OS_MACOSX)
 void InstallStyleSheet(const QString &filename) {
   QFile file(filename);
   file.open(QFile::ReadOnly);
   qApp->setStyleSheet(QLatin1String(file.readAll()));
 }
+#endif
 
 // Use QTextStream to read UTF16 text -- we can't use ifstream,
 // since ifstream cannot handle Wide character.
@@ -276,21 +279,30 @@ UserDictionaryImporter::TextLineIteratorInterface *CreateTextLineIterator(
 }  // namespace
 
 DictionaryTool::DictionaryTool(QWidget *parent)
-    : QMainWindow(parent), dialog_(NULL),
+    : QMainWindow(parent),
+      import_dialog_(NULL), find_dialog_(NULL),
       storage_(
           new UserDictionaryStorage(
               UserDictionaryUtil::GetUserDictionaryFileName())),
       current_dic_id_(0), modified_(false), monitoring_user_edit_(false),
       window_title_(tr("Mozc")),
       dic_menu_(new QMenu),
-      new_action_(NULL), rename_action_(NULL), delete_action_(NULL),
+      new_action_(NULL), rename_action_(NULL),
+      delete_action_(NULL), find_action_(NULL),
       import_create_action_(NULL), import_append_action_(NULL),
       export_action_(NULL), import_default_ime_action_(NULL),
-      session_(new client::Session),
-      is_available_(true) {
+      client_(client::ClientFactory::NewClient()),
+      is_available_(true),
+      max_entry_size_(UserDictionaryStorage::max_entry_size()) {
   setupUi(this);
 
-  session_->set_timeout(kSessionTimeout);
+  // Create and set up ImportDialog object.
+  import_dialog_ = new ImportDialog(this);
+
+  // Create and set up FindDialog object.
+  find_dialog_ = new FindDialog(this, dic_content_);
+
+  client_->set_timeout(kSessionTimeout);
 
   if (!storage_->Load()) {
     LOG(WARNING) << "UserDictionaryStorage::Load() failed";
@@ -400,6 +412,7 @@ DictionaryTool::DictionaryTool(QWidget *parent)
   new_action_    = dic_menu_->addAction(tr("New dictionary..."));
   rename_action_ = dic_menu_->addAction(tr("Rename dictionary..."));
   delete_action_ = dic_menu_->addAction(tr("Delete dictionary"));
+  find_action_   = dic_menu_->addAction(tr("Find..."));
   dic_menu_->addSeparator();
   import_create_action_ =
       dic_menu_->addAction(tr("Import as new dictionary..."));
@@ -422,6 +435,8 @@ DictionaryTool::DictionaryTool(QWidget *parent)
           this, SLOT(RenameDictionary()));
   connect(delete_action_, SIGNAL(triggered()),
           this, SLOT(DeleteDictionary()));
+  connect(find_action_, SIGNAL(triggered()),
+          find_dialog_, SLOT(show()));
   connect(import_create_action_, SIGNAL(triggered()),
           this, SLOT(ImportAndCreateDictionary()));
   connect(import_append_action_, SIGNAL(triggered()),
@@ -450,9 +465,6 @@ DictionaryTool::DictionaryTool(QWidget *parent)
   connect(dic_list_, SIGNAL(itemSelectionChanged()),
           this, SLOT(OnDictionarySelectionChanged()));
 
-  // Create and set up ImportDialog object.
-  dialog_ = new ImportDialog(this);
-
   // Initialize the list widget for a list of dictionaries.
   InitDictionaryList();
   if (dic_list_->count() != 0) {
@@ -575,6 +587,8 @@ void DictionaryTool::OnDictionarySelectionChanged() {
   } else {
     current_dic_id_ = dic_info.id;
     SetupDicContentEditor(dic_info);
+      max_entry_size_ = UserDictionaryStorage::max_entry_size();
+    LOG(INFO) << "Mac entry size: " << max_entry_size_;
   }
 }
 
@@ -629,8 +643,7 @@ void DictionaryTool::SetupDicContentEditor(
   // Update state of other GUI components.
   UpdateUIStatus();
 
-  const bool dictionary_is_full =
-      dic_content_->rowCount() >= UserDictionaryStorage::max_entry_size();
+  const bool dictionary_is_full = dic_content_->rowCount() >= max_entry_size_;
   new_word_button_->setEnabled(!dictionary_is_full);
 
   modified_ = false;
@@ -723,16 +736,16 @@ void DictionaryTool::ImportAndCreateDictionary() {
   }
 
   // Get necessary information from the user.
-  if (dialog_->ExecInCreateMode() != QDialog::Accepted) {
+  if (import_dialog_->ExecInCreateMode() != QDialog::Accepted) {
     LOG(WARNING) << "Cancele by the user.";
     return;
   }
 
   ImportHelper(0,   // dic_id == 0 means that "CreateNewDictonary" mode
-               dialog_->dic_name().toStdString(),
-               dialog_->file_name().toStdString(),
-               dialog_->ime_type(),
-               dialog_->encoding_type());
+               import_dialog_->dic_name().toStdString(),
+               import_dialog_->file_name().toStdString(),
+               import_dialog_->ime_type(),
+               import_dialog_->encoding_type());
 }
 
 void DictionaryTool::ImportAndAppendDictionary() {
@@ -744,24 +757,23 @@ void DictionaryTool::ImportAndAppendDictionary() {
     return;
   }
 
-  const int max_size = UserDictionaryStorage::max_entry_size();
-  if (dic_content_->rowCount() >= max_size) {
+  if (dic_content_->rowCount() >= max_entry_size_) {
     QMessageBox::critical(this, window_title_,
                           tr("You can't have more than %1 "
-                             "words in one dictionary.").arg(max_size));
+                             "words in one dictionary.").arg(max_entry_size_));
     return;
   }
 
-  if (dialog_->ExecInAppendMode() != QDialog::Accepted) {
+  if (import_dialog_->ExecInAppendMode() != QDialog::Accepted) {
     LOG(WARNING) << "Cancele by the user.";
     return;
   }
 
   ImportHelper(dic_info.id,
                dic_info.item->text().toStdString(),
-               dialog_->file_name().toStdString(),
-               dialog_->ime_type(),
-               dialog_->encoding_type());
+               import_dialog_->file_name().toStdString(),
+               import_dialog_->ime_type(),
+               import_dialog_->encoding_type());
 }
 
 void DictionaryTool::ReportImportError(UserDictionaryImporter::ErrorType error,
@@ -964,13 +976,11 @@ void DictionaryTool::ExportDictionary() {
 
 void DictionaryTool::AddWord() {
   const int row = dic_content_->rowCount();
-  const int max_size =
-      static_cast<int>(UserDictionaryStorage::max_entry_size());
-  if (row >= max_size) {
+  if (row >= max_entry_size_) {
     QMessageBox::information(
         this, window_title_,
         tr("You can't have more than %1 words in one dictionary.").arg(
-            max_size));
+            max_entry_size_));
     return;
   }
 
@@ -980,7 +990,7 @@ void DictionaryTool::AddWord() {
   dic_content_->setItem(row, 2, new QTableWidgetItem(default_pos_));
   dic_content_->setItem(row, 3, new QTableWidgetItem(""));
 
-  if (row + 1 >= max_size) {
+  if (row + 1 >= max_entry_size_) {
     new_word_button_->setEnabled(false);
   }
 
@@ -1036,7 +1046,7 @@ void DictionaryTool::DeleteWord() {
   setUpdatesEnabled(true);
   dic_content_->setEnabled(true);
 
-  if (dic_content_->rowCount() < UserDictionaryStorage::max_entry_size()) {
+  if (dic_content_->rowCount() < max_entry_size_) {
     new_word_button_->setEnabled(true);
   }
 
@@ -1366,20 +1376,20 @@ void DictionaryTool::SaveAndReloadServer() {
 
   // If server is not running, we don't need to
   // execute Reload command.
-  if (!session_->PingServer()) {
+  if (!client_->PingServer()) {
     LOG(WARNING) << "Server is not running. Do nothing";
     return;
   }
 
   // Update server version if need be.
-  if (!session_->CheckVersionOrRestartServer()) {
+  if (!client_->CheckVersionOrRestartServer()) {
     LOG(ERROR) << "CheckVersionOrRestartServer failed";
     return;
   }
 
   // We don't show any dialog even when an error happens, since
   // dictionary serialization is finished correctly.
-  if (!session_->Reload()) {
+  if (!client_->Reload()) {
     LOG(ERROR) << "Reload command failed";
   }
 }
@@ -1438,8 +1448,7 @@ void DictionaryTool::UpdateUIStatus() {
 
   const bool is_enable_new_word =
       dic_list_->count() > 0 &&
-      dic_content_->rowCount() <
-      static_cast<int>(UserDictionaryStorage::max_entry_size());
+      dic_content_->rowCount() < max_entry_size_;
 
   new_word_button_->setEnabled(is_enable_new_word);
   delete_word_button_->setEnabled(dic_content_->rowCount() > 0);
