@@ -38,7 +38,9 @@
 
 #include "base/mac_util.h"
 #include "base/util.h"
-#include "client/session_mock.h"
+#include "client/client_mock.h"
+#include "languages/global_language_spec.h"
+#include "languages/japanese/lang_dep_spec.h"
 #include "renderer/renderer_interface.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
@@ -61,7 +63,7 @@
   return self;
 }
 
-- (void)rendererClicked:(NSData *)data {
+- (void)sendData:(NSData *)data {
   // Do nothing
 }
 - (void)outputResult:(NSData *)data {
@@ -240,6 +242,7 @@ class MockRenderer : public mozc::renderer::RendererInterface {
 class GoogleJapaneseInputControllerTest : public testing::Test {
  protected:
   void SetUp() {
+    mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(&spec_);
     pool_ = [[NSAutoreleasePool alloc] init];
     mock_server_ = [[MockIMKServer alloc] init];
     mock_client_ = [[MockClient alloc] init];
@@ -259,9 +262,10 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
   void TearDown() {
     [controller_ release];
     [mock_server_ release];
-    // SessionMock and MockRenderer are released during the release of
+    // mock_mozc_client and mock_renderer are released during the release of
     // |controller_|.
     [pool_ release];
+    mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(NULL);
   }
 
   void SetUpController() {
@@ -270,8 +274,8 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
                           delegate:nil
                             client:mock_client_];
     mock_server_.expectedController = controller_;
-    mock_session_ = new mozc::client::SessionMock;
-    controller_.session = mock_session_;
+    mock_mozc_client_ = new mozc::client::ClientMock;
+    controller_.mozcClient = mock_mozc_client_;
     mock_renderer_ = new MockRenderer;
     controller_.renderer = mock_renderer_;
   }
@@ -282,7 +286,7 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
     SetUpController();
   }
 
-  mozc::client::SessionMock *mock_session_;
+  mozc::client::ClientMock *mock_mozc_client_;
   MockClient *mock_client_;
   MockRenderer *mock_renderer_;
 
@@ -291,6 +295,7 @@ class GoogleJapaneseInputControllerTest : public testing::Test {
  private:
   MockIMKServer *mock_server_;
   NSAutoreleasePool *pool_;
+  mozc::japanese::LangDepSpecJapanese spec_;
 };
 
 // Because preedit has NSMarkedClauseSegment attribute which has to
@@ -540,14 +545,14 @@ TEST_F(GoogleJapaneseInputControllerTest, SwitchModeToDirect) {
   mozc::commands::Output output;
   output.mutable_result()->set_type(mozc::commands::Result::STRING);
   output.mutable_result()->set_value("foo");
-  mock_session_->set_output_SendKey(output);
+  mock_mozc_client_->set_output_SendKey(output);
 
   [controller_ switchModeToDirect:mock_client_];
   EXPECT_EQ(mozc::commands::DIRECT, controller_.mode);
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
-  EXPECT_TRUE(mock_session_->called_SendKey().has_special_key());
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendKey"));
+  EXPECT_TRUE(mock_mozc_client_->called_SendKey().has_special_key());
   EXPECT_EQ(mozc::commands::KeyEvent::OFF,
-            mock_session_->called_SendKey().special_key());
+            mock_mozc_client_->called_SendKey().special_key());
   EXPECT_EQ(1, [mock_client_ getCounter:"insertText:replacementRange:"]);
   EXPECT_TRUE([@"foo" isEqualToString:mock_client_.insertedText]);
   EXPECT_EQ(0, [[controller_ composedString:nil] length]);
@@ -559,32 +564,32 @@ TEST_F(GoogleJapaneseInputControllerTest, SwitchModeInternal) {
   controller_.mode = mozc::commands::DIRECT;
   [controller_ switchModeInternal:mozc::commands::HIRAGANA];
   EXPECT_EQ(mozc::commands::HIRAGANA, controller_.mode);
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
-  EXPECT_TRUE(mock_session_->called_SendKey().has_special_key());
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendKey"));
+  EXPECT_TRUE(mock_mozc_client_->called_SendKey().has_special_key());
   EXPECT_EQ(mozc::commands::KeyEvent::ON,
-            mock_session_->called_SendKey().special_key());
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+            mock_mozc_client_->called_SendKey().special_key());
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::SWITCH_INPUT_MODE,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
   EXPECT_EQ(mozc::commands::HIRAGANA,
-            mock_session_->called_SendCommand().composition_mode());
+            mock_mozc_client_->called_SendCommand().composition_mode());
 
   // Switch from HIRAGANA to KATAKANA.  Just sending mode switching command.
   controller_.mode = mozc::commands::HIRAGANA;
   [controller_ switchModeInternal:mozc::commands::HALF_KATAKANA];
   EXPECT_EQ(mozc::commands::HALF_KATAKANA, controller_.mode);
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
-  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(2, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::SWITCH_INPUT_MODE,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
   EXPECT_EQ(mozc::commands::HALF_KATAKANA,
-            mock_session_->called_SendCommand().composition_mode());
+            mock_mozc_client_->called_SendCommand().composition_mode());
 
   // going to same mode does not cause sendcommand
   [controller_ switchModeInternal:mozc::commands::HALF_KATAKANA];
   EXPECT_EQ(mozc::commands::HALF_KATAKANA, controller_.mode);
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendKey"));
-  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(2, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 }
 
 TEST_F(GoogleJapaneseInputControllerTest, SwitchDisplayMode) {
@@ -626,11 +631,11 @@ TEST_F(GoogleJapaneseInputControllerTest, handleConfig) {
   config.set_preedit_method(mozc::config::Config::KANA);
   config.set_yen_sign_character(mozc::config::Config::BACKSLASH);
   config.set_use_japanese_layout(true);
-  mock_session_->SetConfig(config);
-  mock_session_->SetBoolFunctionReturn("GetConfig", true);
+  mock_mozc_client_->SetConfig(config);
+  mock_mozc_client_->SetBoolFunctionReturn("GetConfig", true);
 
   [controller_ handleConfig];
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("GetConfig"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("GetConfig"));
   EXPECT_EQ(KANA, controller_.keyCodeMap.inputMode);
   EXPECT_EQ(mozc::config::Config::BACKSLASH, controller_.yenSignCharacter);
   EXPECT_EQ(1, [mock_client_ getCounter:"overrideKeyboardWithKeyboardNamed:"]);
@@ -650,9 +655,9 @@ TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndo) {
   // Because of special hack for Eisu/Kana keys, it returns YES.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -660,11 +665,11 @@ TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
 }
 
 TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndoTimeOver) {
@@ -678,9 +683,9 @@ TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndoTimeOver) {
   // Because of special hack for Eisu/Kana keys, it returns YES.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 
   // Sleep more than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 * 2.0);
@@ -688,9 +693,9 @@ TEST_F(GoogleJapaneseInputControllerTest, DoubleTapKanaUndoTimeOver) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 }
 
 TEST_F(GoogleJapaneseInputControllerTest, SingleAndDoubleTapKanaUndo) {
@@ -704,9 +709,9 @@ TEST_F(GoogleJapaneseInputControllerTest, SingleAndDoubleTapKanaUndo) {
   // Because of special hack for Eisu/Kana keys, it returns YES.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 
   // Sleep more than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 * 2.0);
@@ -714,9 +719,9 @@ TEST_F(GoogleJapaneseInputControllerTest, SingleAndDoubleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -724,11 +729,11 @@ TEST_F(GoogleJapaneseInputControllerTest, SingleAndDoubleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
 }
 
 TEST_F(GoogleJapaneseInputControllerTest, TripleTapKanaUndo) {
@@ -742,9 +747,9 @@ TEST_F(GoogleJapaneseInputControllerTest, TripleTapKanaUndo) {
   // Because of special hack for Eisu/Kana keys, it returns YES.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -752,11 +757,11 @@ TEST_F(GoogleJapaneseInputControllerTest, TripleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -764,11 +769,11 @@ TEST_F(GoogleJapaneseInputControllerTest, TripleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(2, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
 }
 
 TEST_F(GoogleJapaneseInputControllerTest, QuadrupleTapKanaUndo) {
@@ -782,9 +787,9 @@ TEST_F(GoogleJapaneseInputControllerTest, QuadrupleTapKanaUndo) {
   // Because of special hack for Eisu/Kana keys, it returns YES.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it did not send command.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -792,11 +797,11 @@ TEST_F(GoogleJapaneseInputControllerTest, QuadrupleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(1, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(1, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -804,11 +809,11 @@ TEST_F(GoogleJapaneseInputControllerTest, QuadrupleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(2, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(2, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());
+            mock_mozc_client_->called_SendCommand().type());
 
   // Sleep less than DoubleTapInterval (sec)
   mozc::Util::Sleep(GetDoubleTapInterval() * 1000.0 / 2.0);
@@ -816,9 +821,9 @@ TEST_F(GoogleJapaneseInputControllerTest, QuadrupleTapKanaUndo) {
   // Send Kana-key.
   EXPECT_EQ(YES, SendKanaKeyEvent(controller_, mock_client_));
   // Check if it did not send key.
-  EXPECT_EQ(0, mock_session_->GetFunctionCallCount("SendKey"));
+  EXPECT_EQ(0, mock_mozc_client_->GetFunctionCallCount("SendKey"));
   // Check if it sent an undo command.
-  EXPECT_EQ(3, mock_session_->GetFunctionCallCount("SendCommand"));
+  EXPECT_EQ(3, mock_mozc_client_->GetFunctionCallCount("SendCommand"));
   EXPECT_EQ(mozc::commands::SessionCommand::UNDO,
-            mock_session_->called_SendCommand().type());  
+            mock_mozc_client_->called_SendCommand().type());  
 }
