@@ -37,7 +37,9 @@
 #include "base/base.h"
 #include "base/const.h"
 #include "base/run_level.h"
-#include "client/session.h"
+#include "client/client.h"
+#include "languages/global_language_spec.h"
+#include "languages/japanese/lang_dep_spec.h"
 
 int main(int argc, char *argv[]) {
   if (!mozc::RunLevel::IsValidClientRunLevel()) {
@@ -46,6 +48,10 @@ int main(int argc, char *argv[]) {
 
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
   InitGoogleWithBreakPad(argv[0], &argc, &argv, false);
+
+  mozc::japanese::LangDepSpecJapanese spec;
+  mozc::language::GlobalLanguageSpec::SetLanguageDependentSpec(&spec);
+
   IMKServer *imkServer = [GoogleJapaneseInputServer getServer];
   if (!imkServer) {
     LOG(FATAL) << mozc::kProductNameInEnglish << " failed to initialize";
@@ -58,8 +64,8 @@ int main(int argc, char *argv[]) {
   // Start the converter server at this time explicitly to prevent the
   // slow-down of the response for initial key event.
   {
-    scoped_ptr<mozc::client::Session> session(new mozc::client::Session);
-    session->PingServer();
+    scoped_ptr<mozc::client::Client> client(new mozc::client::Client);
+    client->PingServer();
   }
   RunApplicationEventLoop();
   [pool drain];
