@@ -33,16 +33,17 @@
 
 #include "base/freelist.h"
 #include "base/util.h"
+#include "config/config.pb.h"
+#include "config/config_handler.h"
+#include "converter/character_form_manager.h"
 #include "converter/converter_interface.h"
-#include "converter/converter_mock.h"
+#include "converter/immutable_converter_interface.h"
 #include "converter/node.h"
 #include "converter/node_allocator.h"
 #include "converter/segments.h"
 #include "dictionary/dictionary_mock.h"
 #include "dictionary/suffix_dictionary.h"
 #include "session/commands.pb.h"
-#include "session/config.pb.h"
-#include "session/config_handler.h"
 #include "testing/base/public/googletest.h"
 #include "testing/base/public/gunit.h"
 
@@ -149,6 +150,7 @@ class DictionaryPredictorTest : public testing::Test {
                                       kRightCapriHiragana,
                                       kCapriKatakana,
                                       Node::SPELLING_CORRECTION);
+
     // "で"
     const char kDe[] = "\xE3\x81\xA7";
 
@@ -221,6 +223,7 @@ TEST_F(DictionaryPredictorTest, OnOffTest) {
   EXPECT_FALSE(predictor.Predict(&segments));
 }
 
+
 TEST_F(DictionaryPredictorTest, BigramTest) {
   Segments segments;
   config::Config config;
@@ -465,9 +468,10 @@ TEST_F(DictionaryPredictorTest,
     // "と"
     MakeSegmentsForSuggestion("\xE3\x81\x82", &segments);
 
-    // history is "てす"
-    const char kHistoryKey[] = "てす";
-    const char kHistoryValue[] = "テス";
+    // "てす"
+    const char kHistoryKey[] = "\xE3\x81\xA6\xE3\x81\x99";
+    // "テス"
+    const char kHistoryValue[] = "\xE3\x83\x86\xE3\x82\xB9";
 
     PrependHistorySegments(kHistoryKey, kHistoryValue,
                            &segments);
@@ -481,8 +485,146 @@ TEST_F(DictionaryPredictorTest,
   }
 }
 
+TEST_F(DictionaryPredictorTest, GetRealtimeCandidateMaxSize) {
+  DictionaryPredictor predictor;
+  Segments segments;
+
+  // GetRealtimeCandidateMaxSize has some heuristics so here we test following
+  // conditions.
+  // - The result must be equal or less than kMaxSize;
+  // - If mixed_conversion is the same, the result of SUGGESTION is
+  //        equal or less than PREDICTION.
+  // - If mixed_conversion is the same, the result of PARTIAL_SUGGESTION is
+  //        equal or less than PARTIAL_PREDICTION.
+  // - Partial version has equal or greater than non-partial version.
+
+  const size_t kMaxSize = 100;
+
+  // non-partial, non-mixed-conversion
+  segments.set_request_type(Segments::PREDICTION);
+  const size_t prediction_no_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, false, kMaxSize);
+  EXPECT_GE(kMaxSize, prediction_no_mixed);
+
+  segments.set_request_type(Segments::SUGGESTION);
+  const size_t suggestion_no_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, false, kMaxSize);
+  EXPECT_GE(kMaxSize, suggestion_no_mixed);
+  EXPECT_LE(suggestion_no_mixed, prediction_no_mixed);
+
+  // non-partial, mixed-conversion
+  segments.set_request_type(Segments::PREDICTION);
+  const size_t prediction_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, prediction_mixed);
+
+  segments.set_request_type(Segments::SUGGESTION);
+  const size_t suggestion_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, suggestion_mixed);
+  EXPECT_EQ(kMaxSize, prediction_mixed + suggestion_mixed);
+
+  // partial, non-mixed-conversion
+  segments.set_request_type(Segments::PARTIAL_PREDICTION);
+  const size_t partial_prediction_no_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, false, kMaxSize);
+  EXPECT_GE(kMaxSize, partial_prediction_no_mixed);
+
+  segments.set_request_type(Segments::PARTIAL_SUGGESTION);
+  const size_t partial_suggestion_no_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, false, kMaxSize);
+  EXPECT_GE(kMaxSize, partial_suggestion_no_mixed);
+  EXPECT_LE(partial_suggestion_no_mixed, partial_prediction_no_mixed);
+
+  // partial, mixed-conversion
+  segments.set_request_type(Segments::PARTIAL_PREDICTION);
+  const size_t partial_prediction_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, partial_prediction_mixed);
+
+  segments.set_request_type(Segments::PARTIAL_SUGGESTION);
+  const size_t partial_suggestion_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, partial_suggestion_mixed);
+  EXPECT_LE(partial_suggestion_mixed, partial_prediction_mixed);
+
+  EXPECT_GE(partial_prediction_no_mixed, prediction_no_mixed);
+  EXPECT_GE(partial_prediction_mixed, prediction_mixed);
+  EXPECT_GE(partial_suggestion_no_mixed, suggestion_no_mixed);
+  EXPECT_GE(partial_suggestion_mixed, suggestion_mixed);
+}
+
+TEST_F(DictionaryPredictorTest, GetRealtimeCandidateMaxSizeForMixed) {
+  DictionaryPredictor predictor;
+  Segments segments;
+  Segment *segment = segments.add_segment();
+
+  const size_t kMaxSize = 100;
+
+  // for short key, try to provide many results as possible
+  segment->set_key("short");
+  segments.set_request_type(Segments::SUGGESTION);
+  const size_t short_suggestion_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, short_suggestion_mixed);
+
+  segments.set_request_type(Segments::PREDICTION);
+  const size_t short_prediction_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, short_prediction_mixed);
+  EXPECT_EQ(kMaxSize, short_prediction_mixed + short_suggestion_mixed);
+
+  // for long key, provide few results
+  segment->set_key("long_request_key");
+  segments.set_request_type(Segments::SUGGESTION);
+  const size_t long_suggestion_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, long_suggestion_mixed);
+  EXPECT_GT(short_suggestion_mixed, long_suggestion_mixed);
+
+  segments.set_request_type(Segments::PREDICTION);
+  const size_t long_prediction_mixed =
+      predictor.GetRealtimeCandidateMaxSize(segments, true, kMaxSize);
+  EXPECT_GE(kMaxSize, long_prediction_mixed);
+  EXPECT_GT(kMaxSize, long_prediction_mixed + long_suggestion_mixed);
+  EXPECT_GT(short_prediction_mixed, long_prediction_mixed);
+}
+
 TEST_F(DictionaryPredictorTest,
        AggregateRealtimeConversion) {
+  // Simple immutable converte mock for the test
+  // TODO(toshiyuki): Implement proper mock under converter directory if needed.
+  class ImmutableConverterMock : public ImmutableConverterInterface {
+   public:
+    ImmutableConverterMock() {
+      Segment *segment = segments_.add_segment();
+      // "わたしのなまえはなかのです"
+      segment->set_key("\xe3\x82\x8f\xe3\x81\x9f\xe3\x81\x97\xe3\x81\xae"
+                       "\xe3\x81\xaa\xe3\x81\xbe\xe3\x81\x88\xe3\x81\xaf"
+                       "\xe3\x81\xaa\xe3\x81\x8b\xe3\x81\xae\xe3\x81\xa7"
+                       "\xe3\x81\x99");
+      Segment::Candidate *candidate = segment->add_candidate();
+      // "私の名前は中野です"
+      candidate->value = "\xe7\xa7\x81\xe3\x81\xae\xe5\x90\x8d\xe5\x89\x8d"
+          "\xe3\x81\xaf\xe4\xb8\xad\xe9\x87\x8e\xe3\x81\xa7\xe3\x81\x99";
+      // "わたしのなまえはなかのです"
+      candidate->key = ("\xe3\x82\x8f\xe3\x81\x9f\xe3\x81\x97\xe3\x81\xae"
+                        "\xe3\x81\xaa\xe3\x81\xbe\xe3\x81\x88\xe3\x81\xaf"
+                        "\xe3\x81\xaa\xe3\x81\x8b\xe3\x81\xae\xe3\x81\xa7"
+                        "\xe3\x81\x99");
+    }
+
+    virtual bool Convert(Segments *segments) const {
+      segments->CopyFrom(segments_);
+      return true;
+    }
+
+   private:
+    Segments segments_;
+  };
+
+  ImmutableConverterMock immutable_converter_mock;
+  ImmutableConverterFactory::SetImmutableConverter(&immutable_converter_mock);
   Segments segments;
   DictionaryPredictor predictor;
   NodeAllocator allocator;
@@ -519,6 +661,8 @@ TEST_F(DictionaryPredictorTest,
   }
 
   EXPECT_EQ(1, segments.conversion_segments_size());
+
+  ImmutableConverterFactory::SetImmutableConverter(NULL);
 }
 
 namespace {
@@ -562,8 +706,10 @@ class TestSuffixDictionary : public DictionaryInterface {
     return result;
   }
 
-  virtual Node *LookupPrefix(const char *str, int size,
-                             NodeAllocatorInterface *allocator) const {
+  virtual Node *LookupPrefixWithLimit(
+      const char *str, int size,
+      const Limit &limit,
+      NodeAllocatorInterface *allocator) const {
     return NULL;
   }
 
@@ -574,6 +720,38 @@ class TestSuffixDictionary : public DictionaryInterface {
 };
 }  // namespace
 
+TEST_F(DictionaryPredictorTest, GetUnigramCandidateCutoffThreshold) {
+  DictionaryPredictor predictor;
+  Segments segments;
+
+  // GetUnigramCandidateCutoffThreshold has some heuristics so here we test
+  // following conditions.
+  // - The result of SUGGESTION is equal or less than PREDICTION.
+  //     - If this condition is broken, expanding suggestion will corrupt
+  //       because SessionConverter::AppendCandidateList doesn't expect
+  //       such situation.
+
+  // non-partial, mixed-conversion
+  segments.set_request_type(Segments::PREDICTION);
+  const size_t prediction_mixed =
+      predictor.GetUnigramCandidateCutoffThreshold(segments, true);
+
+  segments.set_request_type(Segments::SUGGESTION);
+  const size_t suggestion_mixed =
+      predictor.GetUnigramCandidateCutoffThreshold(segments, true);
+  EXPECT_LE(suggestion_mixed, prediction_mixed);
+
+  // non-partial, non-mixed-conversion
+  segments.set_request_type(Segments::PREDICTION);
+  const size_t prediction_no_mixed =
+      predictor.GetUnigramCandidateCutoffThreshold(segments, false);
+
+  segments.set_request_type(Segments::SUGGESTION);
+  const size_t suggestion_no_mixed =
+      predictor.GetUnigramCandidateCutoffThreshold(segments, false);
+  EXPECT_LE(suggestion_no_mixed, prediction_no_mixed);
+}
+
 TEST_F(DictionaryPredictorTest,
        AggregateSuffixPrediction) {
   TestSuffixDictionary dictionary;
@@ -630,6 +808,7 @@ TEST_F(DictionaryPredictorTest,
   SuffixDictionaryFactory::SetSuffixDictionary(NULL);
 }
 
+
 TEST_F(DictionaryPredictorTest, GetHistoryKeyAndValue) {
   Segments segments;
   DictionaryPredictor predictor;
