@@ -37,9 +37,9 @@
 #include "base/config_file_stream.h"
 #include "base/singleton.h"
 #include "base/util.h"
+#include "config/config_handler.h"
+#include "config/config.pb.h"
 #include "session/commands.pb.h"
-#include "session/config_handler.h"
-#include "session/config.pb.h"
 #include "session/internal/keymap.h"
 #include "session/key_event_normalizer.h"
 #include "session/key_parser.h"
@@ -187,7 +187,7 @@ class ImeSwitchUtilImpl {
   void ReloadFromFile(const string &filename) {
     scoped_ptr<istream> ifs(ConfigFileStream::Open(filename));
     if (ifs.get() == NULL) {
-      DCHECK(false) << "could not open file: " << filename;
+      DLOG(FATAL) << "could not open file: " << filename;
       return;
     }
     return ReloadFromStream(ifs.get());
