@@ -40,8 +40,8 @@
 #include "base/protobuf/message.h"
 #include "base/singleton.h"
 #include "base/util.h"
+#include "config/config.pb.h"
 #include "session/commands.pb.h"
-#include "session/config.pb.h"
 #include "session/ime_switch_util.h"
 #include "unix/ibus/config_util.h"
 #include "unix/ibus/engine_registrar.h"
@@ -51,10 +51,10 @@
 
 #ifdef OS_CHROMEOS
 // use standalone (in-process) session
-#include "unix/ibus/session.h"
+#include "unix/ibus/client.h"
 #else
 // use server/client session
-#include "client/session.h"
+#include "client/client.h"
 #endif
 
 namespace {
@@ -164,6 +164,21 @@ void MozcEngineInstanceInit(GTypeInstance *instance, gpointer klass) {
 }  // namespace
 
 namespace mozc {
+namespace {
+bool IsModifierToBeSentOnKeyUp(const commands::KeyEvent &key_event) {
+  if (key_event.modifier_keys_size() == 0) {
+    return false;
+  }
+
+  if (key_event.modifier_keys_size() == 1 &&
+      key_event.modifier_keys(0) == commands::KeyEvent::CAPS) {
+    return false;
+  }
+
+  return true;
+}
+}  // namespace
+
 namespace ibus {
 
 // Returns an IBusText composed from |preedit| to render preedit text.
@@ -283,9 +298,9 @@ MozcEngine::MozcEngine()
     : last_sync_time_(GetTime()),
       key_translator_(new KeyTranslator),
 #ifdef OS_CHROMEOS
-      session_(new ibus::Session),
+      client_(new ibus::Client),
 #else
-      session_(new client::Session),
+      client_(client::ClientFactory::NewClient()),
 #endif
       prop_root_(NULL),
       prop_composition_mode_(NULL),
@@ -395,7 +410,7 @@ MozcEngine::MozcEngine()
   // Currently client capability is fixed.
   commands::Capability capability;
   capability.set_text_deletion(commands::Capability::DELETE_PRECEDING_TEXT);
-  session_->set_client_capability(capability);
+  client_->set_client_capability(capability);
   // TODO(yusukes): write a unit test to check if the capability is set
   // as expected.
 }
@@ -435,7 +450,7 @@ void MozcEngine::CandidateClicked(
   commands::SessionCommand command;
   command.set_type(commands::SessionCommand::SELECT_CANDIDATE);
   command.set_id(id);
-  session_->SendCommand(command, &output);
+  client_->SendCommand(command, &output);
   UpdateAll(engine, output);
 }
 
@@ -459,7 +474,7 @@ void MozcEngine::Enable(IBusEngine *engine) {
   ignore_reset_for_deletion_range_workaround_ = false;
 
   // Launch mozc_server
-  session_->EnsureConnection();
+  client_->EnsureConnection();
   UpdatePreeditMethod();
 
   // When ibus-mozc is disabled by the "next input method" hot key, ibus-daemon
@@ -478,7 +493,9 @@ void MozcEngine::FocusOut(IBusEngine *engine) {
   // Stop ignoring "reset" signal.  See ProcessKeyevent().
   ignore_reset_for_deletion_range_workaround_ = false;
 
-  RevertSession(engine);
+  // Do not call SubmitSession or RevertSession. Preedit string will commit on
+  // Focus Out event automatically by ibus_engine_update_preedit_text_with_mode
+  // which called in UpdatePreedit method.
   SyncData(false);
 }
 
@@ -547,58 +564,13 @@ gboolean MozcEngine::ProcessKeyEvent(
   key.set_mode(current_composition_mode_);
 
   commands::Output output;
-  if (!session_->SendKey(key, &output)) {
+  if (!client_->SendKey(key, &output)) {
     LOG(ERROR) << "SendKey failed";
     return FALSE;
   }
 
   VLOG(2) << output.DebugString();
 
-  if (output.has_deletion_range() &&
-      output.deletion_range().offset() < 0 &&
-      output.deletion_range().offset() + output.deletion_range().length() >=
-          0) {
-    // Delete some characters of preceding text.  We really want to use
-    // ibus_engine_delete_surrounding_text(), but it does not work on many
-    // applications, e.g. Chrome, Firefox.  So we currently forward backspaces
-    // to application.
-    // NOTE: There are some workarounds.  They must be maintained
-    // continuosly.
-    // NOTE: It cannot delete range of characters not adjacent to the left side
-    // of cursor.  If the range is not adjacent to the cursor, ignore it.  If
-    // the range contains the cursor, only characters on the left side of cursor
-    // are deleted.
-    const int length = -output.deletion_range().offset();
-
-    // Some applications, e.g. Chrome, delete preedit character when we forward
-    // a backspace.  We can avoid it by hiding preedit before forwarding
-    // backspaces.
-    ibus_engine_hide_preedit_text(engine);
-
-    // Forward backspaces.
-    for (size_t i = 0; i < length; ++i) {
-      ibus_engine_forward_key_event(
-          engine, IBUS_BackSpace, kBackSpaceKeyCode, 0);
-    }
-
-    // This is a workaround.  Some applications (e.g. Chrome, etc.) send
-    // strange signals such as "focus_out" and "focus_in" after forwarding
-    // Backspace key event.  However, such strange signals are not sent if we
-    // forward another key event after backspaces.  Here we choose Shift L,
-    // which is one of the most quiet key event.
-    ibus_engine_forward_key_event(engine, IBUS_Shift_L, kShiftLeftKeyCode, 0);
-
-    // This is also a workaround.  Some applications (e.g. gedit, etc.) send a
-    // "reset" signal after forwarding backspaces.  This signal is sent after
-    // this ProcessKeyEvent() returns, and we want to avoid reverting the
-    // session.
-    // See also Reset(), which calls RevertSession() only if
-    // ignore_reset_for_deletion_range_workaround_ is false.  This flag is
-    // reset on Disable(), FocusOut(), and ProcessKeyEvent().  Reset() also
-    // reset this flag, i.e. this flag does not work more than once.
-    ignore_reset_for_deletion_range_workaround_ = true;
-  }
-
   UpdateAll(engine, output);
 
   const bool consumed = output.consumed();
@@ -612,14 +584,16 @@ void MozcEngine::SetCompositionMode(
   if (composition_mode == commands::DIRECT) {
     // Commit a preedit string.
     command.set_type(commands::SessionCommand::SUBMIT);
-    session_->SendCommand(command, &output);
-    UpdateAll(engine, output);
+    client_->SendCommand(command, &output);
   } else {
     command.set_type(commands::SessionCommand::SWITCH_INPUT_MODE);
     command.set_composition_mode(composition_mode);
-    session_->SendCommand(command, &output);
+    client_->SendCommand(command, &output);
+    // To esacpe infinity loop, remove mode entry.
+    output.clear_mode();
   }
   current_composition_mode_ = composition_mode;
+  UpdateAll(engine, output);
 }
 
 void MozcEngine::PropertyActivate(IBusEngine *engine,
@@ -636,7 +610,7 @@ void MozcEngine::PropertyActivate(IBusEngine *engine,
             reinterpret_cast<const MozcEngineToolProperty*>(
                 g_object_get_data(G_OBJECT(prop), kGObjectDataKey));
         DCHECK(entry->mode);
-        if (!session_->LaunchTool(entry->mode, "")) {
+        if (!client_->LaunchTool(entry->mode, "")) {
           LOG(ERROR) << "cannot launch: " << entry->mode;
         }
         return;
@@ -709,7 +683,15 @@ void MozcEngine::SetCursorLocation(IBusEngine *engine,
                                    gint y,
                                    gint w,
                                    gint h) {
-  // We can ignore the signal.
+  commands::Output output;
+  commands::SessionCommand command;
+  command.set_type(commands::SessionCommand::SEND_CARET_LOCATION);
+  commands::Rectangle *caret_rectangle = command.mutable_caret_rectangle();
+  caret_rectangle->set_x(x);
+  caret_rectangle->set_y(y);
+  caret_rectangle->set_width(w);
+  caret_rectangle->set_height(h);
+  client_->SendCommand(command, &output);
 }
 
 GType MozcEngine::GetType() {
@@ -847,7 +829,7 @@ bool MozcEngine::ProcessModifiers(
     return false;
   }
   // Clear modifier data just in case if |key| has no modifier keys.
-  if (key->modifier_keys_size() == 0) {
+  if (!IsModifierToBeSentOnKeyUp(*key)) {
     currently_pressed_modifiers->clear();
     modifiers_to_be_sent->clear();
   }
@@ -855,6 +837,7 @@ bool MozcEngine::ProcessModifiers(
 }
 
 bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
+  UpdateDeletionRange(engine, output);
   UpdateResult(engine, output);
   UpdatePreedit(engine, output);
   UpdateCandidates(engine, output);
@@ -862,6 +845,56 @@ bool MozcEngine::UpdateAll(IBusEngine *engine, const commands::Output &output) {
     UpdateCompositionMode(engine, output.mode());
   }
   LaunchTool(output);
+  ExecuteCallback(engine, output);
+  return true;
+}
+
+bool MozcEngine::UpdateDeletionRange(IBusEngine *engine,
+                                     const commands::Output &output) {
+  if (output.has_deletion_range() &&
+      output.deletion_range().offset() < 0 &&
+      output.deletion_range().offset() + output.deletion_range().length() >=
+          0) {
+    // Delete some characters of preceding text.  We really want to use
+    // ibus_engine_delete_surrounding_text(), but it does not work on many
+    // applications, e.g. Chrome, Firefox.  So we currently forward backspaces
+    // to application.
+    // NOTE: There are some workarounds.  They must be maintained
+    // continuosly.
+    // NOTE: It cannot delete range of characters not adjacent to the left side
+    // of cursor.  If the range is not adjacent to the cursor, ignore it.  If
+    // the range contains the cursor, only characters on the left side of cursor
+    // are deleted.
+    const int length = -output.deletion_range().offset();
+
+    // Some applications, e.g. Chrome, delete preedit character when we forward
+    // a backspace.  We can avoid it by hiding preedit before forwarding
+    // backspaces.
+    ibus_engine_hide_preedit_text(engine);
+
+    // Forward backspaces.
+    for (size_t i = 0; i < length; ++i) {
+      ibus_engine_forward_key_event(
+          engine, IBUS_BackSpace, kBackSpaceKeyCode, 0);
+    }
+
+    // This is a workaround.  Some applications (e.g. Chrome, etc.) send
+    // strange signals such as "focus_out" and "focus_in" after forwarding
+    // Backspace key event.  However, such strange signals are not sent if we
+    // forward another key event after backspaces.  Here we choose Shift L,
+    // which is one of the most quiet key event.
+    ibus_engine_forward_key_event(engine, IBUS_Shift_L, kShiftLeftKeyCode, 0);
+
+    // This is also a workaround.  Some applications (e.g. gedit, etc.) send a
+    // "reset" signal after forwarding backspaces.  This signal is sent after
+    // this ProcessKeyEvent() returns, and we want to avoid reverting the
+    // session.
+    // See also Reset(), which calls RevertSession() only if
+    // ignore_reset_for_deletion_range_workaround_ is false.  This flag is
+    // reset on Disable(), FocusOut(), and ProcessKeyEvent().  Reset() also
+    // reset this flag, i.e. this flag does not work more than once.
+    ignore_reset_for_deletion_range_workaround_ = true;
+  }
   return true;
 }
 
@@ -885,7 +918,12 @@ bool MozcEngine::UpdatePreedit(IBusEngine *engine,
     return true;
   }
   IBusText *text = ComposePreeditText(output.preedit());
+#if IBUS_CHECK_VERSION(1, 3, 99)
+  ibus_engine_update_preedit_text_with_mode(engine, text, CursorPos(output),
+                                            TRUE, IBUS_ENGINE_PREEDIT_COMMIT);
+#else
   ibus_engine_update_preedit_text(engine, text, CursorPos(output), TRUE);
+#endif
   // |text| is released by ibus_engine_update_preedit_text.
   return true;
 }
@@ -932,43 +970,7 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
 
   for (int i = 0; i < candidates.candidate_size(); ++i) {
     const commands::Candidates::Candidate &candidate = candidates.candidate(i);
-
-#ifdef OS_CHROMEOS
-    const bool has_description = candidate.has_annotation() &&
-                                 candidate.annotation().has_description();
-    IBusText *text = NULL;
-    if (has_description) {
-      // |kDelimiter| divides a candidate and an annotation.
-      static const char kDelimiter[] = " ";
-
-      // Append an annotation to a candidate word. Both are separated
-      // by |kDelimiter|.
-      text = ibus_text_new_from_string(
-                 (candidate.value() + kDelimiter +
-                  candidate.annotation().description()).c_str());
-
-      // The candidate window on Chrome OS will know
-      // the start index of an annotation by specific number (e.g. 0x888888).
-      // TODO(nhiroki): We should modify the way when iBus supports annotations.
-      const guint kAnnotationForegroundColor = 0x888888;
-
-      // Insert an attribute. It incidates annotation's
-      // start and end index.
-      const guint start = Util::CharsLen(candidate.value() + kDelimiter);
-      const guint end = start +
-          Util::CharsLen(candidate.annotation().description());
-      ibus_text_append_attribute(text,
-                                 IBUS_ATTR_TYPE_FOREGROUND,
-                                 kAnnotationForegroundColor,
-                                 start,
-                                 end);
-    } else {
-      text = ibus_text_new_from_string(candidate.value().c_str());
-    }
-#else
     IBusText *text = ibus_text_new_from_string(candidate.value().c_str());
-#endif
-
     ibus_lookup_table_append_candidate(table, text);
     // |text| is released by ibus_engine_update_lookup_table along with |table|.
 
@@ -986,6 +988,34 @@ bool MozcEngine::UpdateCandidates(IBusEngine *engine,
     // |label| is released by ibus_engine_update_lookup_table along with
     // |table|.
   }
+
+#if defined(OS_CHROMEOS) and IBUS_CHECK_VERSION(1, 3, 99)
+  // The function ibus_serializable_set_attachment() had been changed
+  // to use GVariant in ibus-1.3.99.
+  // https://github.com/ibus/ibus/commit/ac9dfac13cef34288440a2ecdf067cd827fb2f8f
+  // But these codes are valid only for ChromeOS since:
+  //  1) IBus's default panel (main.py) does not support the attachment.
+  //  2) Ubuntu 10.10 uses ibus-1.3.99, but the version of ibus it uses is
+  //     very old.
+  // If we only use IBUS_CHECK_VERSION, ibus-mozc does not compile on
+  // Ubuntu 10.10.
+  // Also we do not use only OS_CHROMEOS, because we will compile ibus-mozc for
+  // ChromeOS on Ubuntu for debugging even if following feature is missed.
+  if (output.has_candidates()) {
+    string buf;
+    output.candidates().SerializeToString(&buf);
+
+    GByteArray *bytearray = g_byte_array_sized_new(buf.length());
+    g_byte_array_append(bytearray,
+        reinterpret_cast<const guint8*>(buf.c_str()), buf.length());
+    GVariant* variant = g_variant_new_from_data(G_VARIANT_TYPE("ay"),
+        bytearray->data, bytearray->len, TRUE,
+        reinterpret_cast<GDestroyNotify>(g_byte_array_unref), bytearray);
+    ibus_serializable_set_attachment(
+        IBUS_SERIALIZABLE(table), "mozc.candidates", variant);
+  }
+#endif
+
   ibus_engine_update_lookup_table(engine, table, TRUE);
   // |table| is released by ibus_engine_update_lookup_table.
 
@@ -1028,13 +1058,13 @@ void MozcEngine::UpdateConfig(const gchar *section,
   }
 
   config::Config mozc_config;
-  session_->GetConfig(&mozc_config);
+  client_->GetConfig(&mozc_config);
   ConfigUtil::SetFieldForName(name, value, &mozc_config);
 
   // Update config1.db.
-  session_->SetConfig(mozc_config);
-  session_->SyncData();  // TODO(yusukes): remove this call?
-  VLOG(2) << "Session::SetConfig() is called: " << name;
+  client_->SetConfig(mozc_config);
+  client_->SyncData();  // TODO(yusukes): remove this call?
+  VLOG(2) << "Client::SetConfig() is called: " << name;
 }
 #endif  // OS_CHROMEOS
 
@@ -1053,7 +1083,7 @@ void MozcEngine::UpdateCompositionMode(
 
 void MozcEngine::UpdatePreeditMethod() {
   config::Config config;
-  if (!session_->GetConfig(&config)) {
+  if (!client_->GetConfig(&config)) {
     LOG(ERROR) << "GetConfig failed";
     return;
   }
@@ -1062,7 +1092,7 @@ void MozcEngine::UpdatePreeditMethod() {
 }
 
 void MozcEngine::SyncData(bool force) {
-  if (session_.get() == NULL) {
+  if (client_.get() == NULL) {
     return;
   }
 
@@ -1071,13 +1101,13 @@ void MozcEngine::SyncData(bool force) {
       (current_time >= last_sync_time_ &&
        current_time - last_sync_time_ >= kSyncDataInterval)) {
     VLOG(1) << "Syncing data";
-    session_->SyncData();
+    client_->SyncData();
     last_sync_time_ = current_time;
   }
 }
 
 bool MozcEngine::LaunchTool(const commands::Output &output) const {
-  if (!session_->LaunchToolWithProtoBuf(output)) {
+  if (!client_->LaunchToolWithProtoBuf(output)) {
     VLOG(2) << output.DebugString() << " Launch Failed";
     return false;
   }
@@ -1092,7 +1122,7 @@ void MozcEngine::RevertSession(IBusEngine *engine) {
   commands::SessionCommand command;
   command.set_type(commands::SessionCommand::REVERT);
   commands::Output output;
-  if (!session_->SendCommand(command, &output)) {
+  if (!client_->SendCommand(command, &output)) {
     LOG(ERROR) << "RevertSession() failed";
     return;
   }
@@ -1104,5 +1134,89 @@ void MozcEngine::RevertSession(IBusEngine *engine) {
   }
 }
 
+bool MozcEngine::ExecuteCallback(IBusEngine *engine,
+                                 const commands::Output &output) {
+  if (!output.has_callback()) {
+    return false;
+  }
+
+  // TODO(nona): Make IBus interface class and add unittest for ibus APIs.
+  if (!output.callback().has_session_command()) {
+    LOG(ERROR) << "callback does not have session_command";
+    return false;
+  }
+
+  const commands::SessionCommand &callback_command =
+      output.callback().session_command();
+
+  if (!callback_command.has_type()) {
+    LOG(ERROR) << "callback_command has no type";
+    return false;
+  }
+
+  commands::SessionCommand session_command;
+  session_command.set_type(callback_command.type());
+
+  switch (callback_command.type()) {
+    case commands::SessionCommand::UNDO:
+      // do nothing.
+      break;
+    case commands::SessionCommand::CONVERT_REVERSE: {
+#if IBUS_CHECK_VERSION(1, 4, 0)
+// ibus_engine_get_surrounding_text is supported by >= 1.4.0
+      IBusText *text = NULL;
+      guint cursor_pos = 0;
+      guint anchor_pos = 0;
+
+      ibus_engine_get_surrounding_text(engine, &text, &cursor_pos,
+                                         &anchor_pos);
+      if (cursor_pos == anchor_pos) {
+        // There are no selection texts.
+        return false;
+      }
+
+      const string surrounding_text(ibus_text_get_text(text));
+      g_object_unref(text);
+
+      string selection_text;
+      const uint32 selection_start = min(cursor_pos, anchor_pos);
+      const uint32 selection_length = abs(cursor_pos - anchor_pos);
+      Util::SubString(surrounding_text,
+                      selection_start,
+                      selection_length,
+                      &selection_text);
+
+      session_command.set_text(selection_text);
+      break;
+#else
+      return false;
+#endif  // IBUS_CHECK_VERSION(1, 4, 0)
+    }
+    default:
+      return false;
+  }
+
+  commands::Output new_output;
+  if (!client_->SendCommand(session_command, &new_output)) {
+    LOG(ERROR) << "Callback Command Failed";
+    return false;
+  }
+
+  if (callback_command.type() == commands::SessionCommand::CONVERT_REVERSE) {
+    // We need remove selection area, but "delete-surrounding-text" API is
+    // unstable. So we send backspace key event once to remove selection area.
+    // To set deletion range as follows, the engine forwards backspace key event
+    // once to the client application via ibus-daemon.
+    commands::DeletionRange *range = new_output.mutable_deletion_range();
+    range->set_offset(-1);
+    range->set_length(1);
+  }
+
+  // Here uses recursion of UpdateAll but it's okay because the converter
+  // ensures that the second output never contains callback.
+  UpdateAll(engine, new_output);
+  return true;
+}
+
 }  // namespace ibus
 }  // namespace mozc
